
export async function getData(data, firstDataRequest) {
	if(firstDataRequest && sessionStorage.getItem(symbol)){
		try {
			let sessionData = JSON.parse(sessionStorage.getItem(symbol));
			if(typeof sessionData !== 'object' || 
				(typeof sessionData === 'object' && (sessionData.length === 0 || Object.keys(sessionData).length === 0))){
				return makeApiRequest(data);
			}

			return JSON.parse(sessionStorage.getItem(symbol));
		} catch (e) {
			return makeApiRequest(data);
		}
	}else{
		return makeApiRequest(data);
	}
}


// Make requests to API
export async function makeApiRequest(data) {
	try {
		let t = a();
		let link = apiURL;
		
		let extraHeader = {};
		let body = JSON.stringify({"data": JSON.stringify(data)});
		if(userType !== 'mt4' && userType !== 'mt5'){
			link = mtsApiURL;
			extraHeader = {
				'Content-Type': 'application/json',
				'apiKey': apiKey
			};
			body = JSON.stringify(data);
		}
		const response = await fetch(link, {
			method: 'POST', // or 'PUT'
			headers: {
            	product: "vt",
				apiVer: "v1",
				product: "vt",
				version: "9.9.9",
				sign: hex_md5(b(t, api_ip)).toUpperCase(),
				ts: t,
				systemType: "h5",
				uuid: uuid,
				model: "H5",
				ip: api_ip,
				...extraHeader
			},
				body: body
			})
			.then((response) => response.json())
			.then((res) => {
				if(userType === 'mt4' || userType === 'mt5'){
					if(res.code == '10100010' || res.code == '10500073'){
						return [];
					}

					let depressData = gzipDecompress(res.obj.data);
					depressData = JSON.parse(depressData);
					return depressData?.list;
				}
				return res.data?.data?.list;
			})
			.catch((error) => {
				return [];
			});
		return response;
	} catch (error) {
		throw new Error(`API request error: ${error.status}`);
		
	}
	
}




export const gzipDecompress = (compressedData) => {
	// Decode base64 data to binary (Uint8Array)
	const binaryData = new Uint8Array(atob(compressedData).split('').map(char => char.charCodeAt(0)));
	// Decompress the binary data using pako.inflate() or pako.ungzip()
	const decompressedData = pako.inflate(binaryData, { to: 'string' });
	// const jsonString = JSON.parse(decompressedData);
	return decompressedData;
}

// Generate a symbol ID from a pair of the coins
export function generateSymbol(exchange, fromSymbol, toSymbol) {
	const short = `${fromSymbol}/${toSymbol}`;
	return {
		short,
		full: `${exchange}:${short}`,
	};
}

export function parseFullSymbol(fullSymbol) {
	return {
		exchange: fullSymbol,
		symbol: fullSymbol,
		type: fullSymbol,
	};
}

export function getNextDailyBarTime(barTime, timeframe) {
	let date1 = new Date(barTime);
	let minitFrame = [1, 5, 15, 30];

	if(minitFrame.includes(Number(timeframe))){
		date1.setMinutes(date1.getMinutes() + Number(timeframe));
	}else if(timeframe === 60){
		date1.setHours(date1.getHours() + 1);
	}else if(timeframe === 240){
		date1.setHours(date1.getHours() + 4);
	}else if(timeframe === '1D'){
		date1.setDate(date1.getDate() + 1);
	}
	else if(timeframe === '1W'){
		date1.setDate(date1.getDate() + 7);
	}
	else {
		date1.setMonth(date1.getMonth() + 1);
	}

	return date1.getTime();
}


export function addTimestampUTC(convertTime, utc) {

	let time = new Date(convertTime);
	// Parse the input timestamp into a Date object
	const inputDate = time;
	// Get the UTC time in milliseconds
	const utcTimeMs = inputDate.getTime();

	// Calculate the UTC+3 time in milliseconds
	const utcPlus3TimeMs = utcTimeMs + utc * 60 * 60 * 1000; // 3 hours in milliseconds
	// Create a new Date object representing the UTC+3 time
	const utcPlus3Date = new Date(utcPlus3TimeMs);

	// Format the UTC+3 timestamp as a string
	time = utcPlus3Date.getTime();

	return time;
}

export function deductTimestampUTC(convertTime, utc) {

	let time = new Date(convertTime);
	// Parse the input timestamp into a Date object
	const inputDate = time;
	// Get the UTC time in milliseconds
	const utcTimeMs = inputDate.getTime();

	// Calculate the UTC+3 time in milliseconds
	const utcPlus3TimeMs = utcTimeMs - utc * 60 * 60 * 1000; // 3 hours in milliseconds
	// Create a new Date object representing the UTC+3 time
	const utcPlus3Date = new Date(utcPlus3TimeMs);

	// Format the UTC+3 timestamp as a string
	time = utcPlus3Date.getTime();

	return time;
}

  